@@ -8,7 +8,7 @@ use PDL::NiceSlice;
 use PDL::Graphics::TriD;
 
 # $PDL::debug_trid=1;
-# $PDL::Graphics::TriD::verbose = 100;
+$PDL::Graphics::TriD::verbose //= 0;
 $im = sequence(640,480)/640.0/480.0;
 $im3 = $im->dummy(0,3);
 
